/**
 * @(#)MaileonContactFiltersService.java
 */
package com.maileon.api.contactfilters;

import com.maileon.api.*;
import com.maileon.api.contactfilters.xml.ContactFilterXmlSerializer;
import com.maileon.api.contactfilters.xml.ContactFiltersXmlSerializer;
import com.maileon.api.utils.PageUtils;
import org.dom4j.Element;

/**
 * The <code>MaileonContactFiltersService</code> client sends operational requests to the <code>ContactFiltersResource</code>.
 *
 */
public class MaileonContactFiltersService extends AbstractMaileonService {

    public static final String SERVICE = "MAILEON ContactFilters";

    /**
     * Constructs a <code>MaileonContactFiltersService</code>.
     *
     * @param config Maileon API-Configuration.
     */
    public MaileonContactFiltersService(MaileonConfiguration config) {
        super(config, SERVICE);
    }

    /**
     * Returns the total count of contact filters in the account. See
     * <a href="http://dev.maileon.com/api/rest-api-1-0/contactfilters/get-contactfilters-count/?lang=en">Maileon API documentation</a>.
     *
     * @throws MaileonException
     */
    public int getContactFiltersCount() throws MaileonException {
        ResponseWrapper response = get("contactfilters/count");
        try {
            return Integer.parseInt(response.getEntityAsXml().getText());
        } catch (Throwable t) {
            throw new MaileonException("Unable to parse contact filter count as integer: " + response.getEntityAsXml().getText(), t);
        }
    }

    /**
     * Retuns a page of contact filters using the provided pagination settings. See
     * <a href="http://dev.maileon.com/api/rest-api-1-0/contactfilters/get-contactfilters/?lang=en">Maileon API documentation</a>.
     *
     * @param pageSize required number of elements in the result page [1..1000].
     * @param pageIndex starts with 1.
     * @return a page containing the returned {@link ContactFilter} objects.
     * @throws MaileonException
     */
    public Page<ContactFilter> getContactFilters(int pageSize, int pageIndex) throws MaileonException {
        QueryParameters qp = new QueryParameters("pageSize", pageSize);
        qp.add("pageIndex", pageIndex);

        ResponseWrapper response = get("contactfilters", qp);

        try {
            Page<ContactFilter> page = PageUtils.createPage(pageIndex, pageSize, response);

            Element xml = response.getEntityAsXml();
            page.setItems(ContactFiltersXmlSerializer.deserialize(xml));

            return page;
        } catch (Exception e) {
            throw new MaileonException("Problem while executing GET on all contact filters", e);
        }
    }

    /**
     * Gets a contacts-filter according to a particular id. See
     * <a href="http://dev.maileon.com/api/rest-api-1-0/contactfilters/get-contactfilter/?lang=en">Maileon API documentation</a>.
     *
     * @param contactFilterId id of the requested filter
     * @return the {@link ContactFilter} instance
     * @throws MaileonException
     */
    public ContactFilter getContactFilter(long contactFilterId) throws MaileonException {
        ResponseWrapper response = get("contactfilters/contactfilter/" + contactFilterId);

        try {
            return ContactFilterXmlSerializer.deserializeSingleElement(response.getEntityAsXml());
        } catch (Exception e) {
            throw new MaileonException("Problem while executing GET on single contact filter", e);
        }
    }

    /**
     * Updates a particular contacts filter with new settings. See
     * <a href="http://dev.maileon.com/api/rest-api-1-0/contactfilters/update-contact-filter/?lang=en">Maileon API documentation</a>.
     *
     * @param contactFilterId id of the required filter
     * @param newFilterData the new data with which the contact filter will be updated. Currently only the name of the underlying filter will be
     * updated
     * @throws MaileonException
     */
    public void updateContactFilter(long contactFilterId, ContactFilter newFilterData) throws MaileonException {
        try {
            post("contactfilters/contactfilter/" + contactFilterId, null, ContactFilterXmlSerializer.serialize(newFilterData).asXML());
        } catch (MaileonException e) {
            throw e;
        } catch (Exception e) {
            throw new MaileonException("Problem while executing POST", e);
        }
    }

    /**
     * Update contacts of a particular contacts filter. See
     * <a href="http://dev.maileon.com/api/rest-api-1-0/contactfilters/refresh-contactfilter-contacts/?lang=en">Maileon API documentation</a>.
     *
     * @param contactFilterId id of the required contacts filter
     * @param time actuality time. The filter is going to be updated if the last actualization has been done before the given time
     * @param asynchronously if true, then update will be queued
     * @throws MaileonException
     */
    public void refreshContactFilterContacts(long contactFilterId, long time, boolean asynchronously) throws MaileonException {
        try {
            QueryParameters qp = new QueryParameters("time", time);
            qp.add("async", asynchronously);

            get("contactfilters/contactfilter/" + contactFilterId + "/refresh", qp);
        } catch (MaileonException e) {
            throw e;
        } catch (Exception e) {
            throw new MaileonException("Problem while executing refreshing filter " + contactFilterId, e);
        }
    }

    /**
     * Update contacts of a particular contacts filter synchronously. See
     * <a href="http://dev.maileon.com/api/rest-api-1-0/contactfilters/refresh-contactfilter-contacts/?lang=en">Maileon API documentation</a>.
     *
     * @param contactFilterId id of the required contacts filter
     * @param time actuality time. The filter is going to be updated if the last actualization has been done before the given time
     * @throws MaileonException
     * @see #refreshContactFilterContacts(long, long, boolean)
     */
    public void refreshContactFilterContacts(long contactFilterId, long time) throws MaileonException {
        refreshContactFilterContacts(contactFilterId, time, false);
    }
}
