/**
 * @(#)Transaction.java
 */
package com.maileon.api.transactions;

import org.apache.commons.io.IOUtils;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Base64;
import java.util.List;

public class Transaction {
    private long type;
    private ContactReference contact;
    private ImportReference importReference;
    private Object content;
    private final List<Attachment> attachments = new ArrayList<>();

    public long getType() {
        return type;
    }

    public void setType(long type) {
        this.type = type;
    }

    public ContactReference getContact() {
        return contact;
    }

    public void setContact(ContactReference contact) {
        this.contact = contact;
    }

    public Object getContent() {
        return content;
    }

    public void setContent(Object content) {
        this.content = content;
    }

    public void setContentFromString(String jsonContent) {
        try {
            setContent(new JSONParser().parse(jsonContent));
        } catch (ParseException e) {
            throw new IllegalArgumentException(e);
        }
    }

    public ImportReference getImportReference() {
        return importReference;
    }

    public void setImportReference(ImportReference importReference) {
        this.importReference = importReference;
    }

    /**
     * Read an attachment from a file and add it to the transaction, using the name of the provided {@link File}
     * as the attachment's file name.
     *
     * @param file the file to read the attachment contents from
     * @param mimetype the mime type of the attachment
     * @throws IOException
     */
    public void addAttachment(File file, String mimetype) throws IOException {
        addAttachment(file.getName(), mimetype, new FileInputStream(file));
    }

    /**
     * Read an attachment from a file and add it to the transaction.
     * @param filename the file name of the attachment to use in the e-mail
     * @param mimetype the mime type of the attachment
     * @param file the file to read the attachment contents from
     * @throws IOException
     */
    public void addAttachment(String filename, String mimetype, File file) throws IOException {
        FileInputStream fis = new FileInputStream(file);
        try {
            addAttachment(filename, mimetype, fis);
        } finally {
            fis.close();
        }
    }

    /**
     * Read an attachment from a stream and add it to the transaction. This method closes the provided input stream.
     * @param filename the file name of the attachment to use in the e-mail
     * @param mimetype the mime type of the attachment
     * @param inputStream the input stream to read the attachment contents from.
     * @throws IOException
     */
    public void addAttachment(String filename, String mimetype, InputStream inputStream) throws IOException {
        try {
            byte[] content = IOUtils.toByteArray(inputStream);
            addAttachment(filename, mimetype, content);
        } finally {
            inputStream.close();
        }
    }

    /**
     * Add an attachment to the transaction.
     * @param filename the file name of the attachment to use in the e-mail
     * @param mimetype the mime type of the attachment
     * @param contents the attachment contents
     */
    public void addAttachment(String filename, String mimetype, byte[] contents) {
        String encodedContents = Base64.getEncoder().encodeToString(contents);
        Attachment attachment = new Attachment(filename, mimetype, encodedContents);
        attachments.add(attachment);
    }

    public List<Attachment> getAttachments() {
        return attachments;
    }
}
