package com.maileon.api.mailings.mailing;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.is;
import static org.junit.Assert.*;

import com.maileon.api.MaileonException;
import com.maileon.api.MaileonNotFoundException;
import com.maileon.api.Page;
import com.maileon.api.mailings.MaileonMailingsService;
import com.maileon.api.mailings.Mailing;
import com.maileon.api.mailings.MailingFields;
import com.maileon.api.mailings.MailingFilterOperators;
import com.maileon.api.mailings.checks.MaileonMailingCheckService;
import com.maileon.api.test.TestProperties;
import org.custommonkey.xmlunit.Diff;
import org.dom4j.Element;
import org.hamcrest.Matchers;
import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

/**
 * Test driver for the mailing service
 *
 * @author Marcus St&auml;nder | Trusted Mails GmbH | <a href="mailto:marcus.staender@trusted-mails.com">marcus.staender@trusted-mails.com</a>
 *
 */
public class MaileonMailingsServiceTest {
    private static MaileonMailingsService service;
    private static MaileonMailingCheckService checkService;
    private static TestProperties testProperties;

    // Test
//    private static final long DEFAULT_MAILING_ID_SEALED = 237;

    // Live
    private static final long DEFAULT_MAILING_ID_SEALED = 8;
    private static final long DEFAULT_MAILING_ID = 1003;
    private static final String DEFAULT_TEST_RECIPIENT = "marcus.beckerle@xqueue.com";
    private static final long DEFAULT_TEST_TARGETGROUP = 82;
    private static final long DEFAULT_NOT_TEST_TARGETGROUP = 20;

    private static final String DEFAULT_NEW_SENDER_ALIAS = "New Sender Alias";
    private static final String DEFAULT_NEW_RECIPIENT_ALIAS = "New Recipient Alias";
    private static final String DEFAULT_NEW_SPEED_LEVEL = "medium";
    private static final String DEFAULT_NEW_TRACKING_STRATEGY = "highest-permission";
    private static final String DEFAULT_NEW_SENDER = "test123@xqueue.com";
    private static final String DEFAULT_NEW_SUBJECT = "newSubject";
    private static final long DEFAULT_NEW_TARGET_GROUP_ID = 2;
    private static final String DEFAULT_NEW_NAME = "NewTestname";

    private static final String MAILING_SUBJECT = "Automated integration test mailing";

    public static Long mailingId;
    public static Long mailingId2;
    public static Long mailingId3;

    @BeforeClass
    public static void setUp() throws MaileonException {
        testProperties = new TestProperties();
        service = new MaileonMailingsService(testProperties.getMaileonConfiguration());
        service.setDebug(testProperties.isDebug());
        checkService = new MaileonMailingCheckService(testProperties.getMaileonConfiguration());
        checkService.setDebug(testProperties.isDebug());

    }

    @Before
    public void initializeTest() {
        mailingId = null;
        mailingId2 = null;
        mailingId3 = null;
    }

    @After
    public void cleanUpStaleTestdata() throws MaileonException {
        service.setDebug(false);

        if (mailingId != null) {
            try {
                service.deleteMailing(mailingId);
            } catch (MaileonException e) {
                e.printStackTrace();
            }
        }

        if (mailingId2 != null) {
            try {
                service.deleteMailing(mailingId2);
            } catch (MaileonException e) {
                e.printStackTrace();
            }
        }

        if (mailingId3 != null) {
            try {
                service.deleteMailing(mailingId3);
            } catch (MaileonException e) {
                e.printStackTrace();
            }
        }

        service.setDebug(testProperties.isDebug());
    }

    @Test
    public void testMailingExists() throws MaileonException {

        String subject = getRandomSubject();
        String name = getRandomName();

        boolean exists = service.existsMailing(name);
        assertFalse(exists);

        mailingId = service.createMailing(name, subject);
        assertNotNull(mailingId);

        exists = service.existsMailing(name);
        assertTrue(exists);
    }

    @Test
    public void testGetMailingByName() throws MaileonException {

        String subject = getRandomSubject();
        String name = getRandomName();

        mailingId = service.createMailing(name, subject);
        assertNotNull(mailingId);

        Long checkedId = service.getMailingId(name);
        assertEquals(mailingId, checkedId);
    }

    @Test
      public void testCreateRegularMailing() throws MaileonException {

        String subject = getRandomSubject();
        String name = getRandomName();

        mailingId = service.createMailing(name, subject);
        assertNotNull(mailingId);

        String apiReportedType = service.getType(mailingId);
        assertEquals(MaileonMailingsService.MailingType.REGULAR.getApiString(), apiReportedType);

    }

    @Test
    public void testSetAndGetMailingLocale() throws MaileonException {

        long mailingId =  476;

        String testLocale = "de";

        service.setLocale(mailingId, testLocale);
        String locale = service.getLocale(mailingId);
        assertEquals(testLocale, locale);

        testLocale = "en";
        service.setLocale(mailingId, testLocale);
        locale = service.getLocale(mailingId);
        assertEquals(locale, testLocale);
    }

    @Test
    public void testGetMailingsBySubject() throws MaileonException {
        String subject = getRandomSubject();
        mailingId = service.createMailing(getRandomName(), subject);

        Page<Mailing> mailingyBySubject = service.getMailingsBySubject(subject, MailingFilterOperators.STRING_OP_CONTAINS, new ArrayList<String>(), 1, 100);
        assertEquals(1, mailingyBySubject.getTotalItems());
        assertThat(mailingyBySubject.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId))));
    }

    @Test
    public void testGetTwoMailingsBySubject() throws MaileonException {
        String subject = getRandomSubject();
        mailingId = service.createMailing(getRandomName(), subject);
        mailingId2 = service.createMailing(getRandomName(), subject);

        Page<Mailing> mailingyBySubject = service.getMailingsBySubject(subject, MailingFilterOperators.STRING_OP_CONTAINS, new ArrayList<String>(), 1, 100);
        assertEquals(2, mailingyBySubject.getTotalItems());
        assertThat(mailingyBySubject.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId))));
        assertThat(mailingyBySubject.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId2))));
    }

    @Test
    public void testGetMailingsByCreator() throws MaileonException {
        List<String> fields = new ArrayList<>();
        fields.add(MailingFields.STATE);
        fields.add(MailingFields.TYPE);
        fields.add(MailingFields.NAME);
        fields.add(MailingFields.SCHEDULE_TIME);

        String randomName = getRandomName();
        String subject = getRandomSubject();
        mailingId = service.createMailing(randomName, subject);

        Page<Mailing> mailings = service.getMailingsByCreatorName("API", MailingFilterOperators.STRING_OP_CONTAINS, fields, 1, 1000);

        assertThat(mailings.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId))));
    }

    @Test
    public void testReplyToAddress() throws Exception {

        mailingId = service.createMailing(getRandomName(), MAILING_SUBJECT);
        Element replyToAddress = service.getReplyToAddress(mailingId);

        String expected = "<reply_to><auto>true</auto><active>true</active><customEmail/></reply_to>";
        Diff diff = new Diff(expected, replyToAddress.asXML());
        assertTrue("XML similar " + diff.toString(), diff.similar());

        boolean active = true;
        boolean auto = false;
        String customEmail = DEFAULT_NEW_SENDER;
        service.setReplyToAddress(mailingId, active, auto, customEmail);

        replyToAddress = service.getReplyToAddress(mailingId);
        expected = "<reply_to><auto>false</auto><active>true</active><customEmail>" + customEmail + "</customEmail></reply_to>";
        diff = new Diff(expected, replyToAddress.asXML());
        assertTrue("XML similar " + diff.toString(), diff.similar());
    }

    @Test
    public void testgetArchiveUrl() throws MaileonException {
        // This test relies on a sent email
        long sealedMailingId = DEFAULT_MAILING_ID_SEALED;

        assertTrue(service.isSealed(sealedMailingId));

        String archiveUrl = service.getArchiveUrl(sealedMailingId);
        assertEquals("http://api.testing.news-mailer.eu/u/archive/Z0dUIfmJCmA", archiveUrl);
    }

    @Test
    public void testCreateAndSetMailing() throws MaileonException {
        int amountOfLinks = 10;
        int amountOfImages = 10;

        SimpleDateFormat format = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        long start = System.currentTimeMillis();
        System.out.println(format.format(System.currentTimeMillis()) + " - Starting creation...");
        mailingId = service.createMailing("API created mailing " + UUID.randomUUID().toString().replace("-", "").substring(0, 10), "Fancy new API-created mailing!");
        System.out.println(format.format(System.currentTimeMillis()) + " - Duration to create: " + (System.currentTimeMillis() - start) + "ms");
        StringBuilder html = new StringBuilder();

        html.append("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\"><html>   <head>      <title>Test</title>	</head>   <body bgcolor=\"#ebebeb\"><ul>");

        for (int i = 0; i < amountOfLinks; i++) {
            html.append("<li>").append("<a href=\"http://www.xqueue.de").append(i).append("\">Link ").append(i).append("</a>").append("</li>");
        }

        for (int i = 0; i < amountOfImages; i++) {
            html.append("<img src=\"http://www.desktopwallpaperhd.net/wallpapers/11/f/wallpaper-widescreen-paper-114575.jpg\" />");
        }

        html.append("</ul></body></html>");

        System.out.println(format.format(System.currentTimeMillis()) + " - Now sending content...");
        service.setHtmlContent(mailingId, html.toString());

        long duration = System.currentTimeMillis() - start;
        System.out.println(format.format(System.currentTimeMillis()) + " - Done. Overall-duration: " + duration + "ms");
    }


    @Test
    public void testGetMailingsBySubjectEquals() throws MaileonException {
        String subject = getRandomSubject();
        String name1 = getRandomName();
        String name2 = getRandomName();
        String name3 = getRandomName();

        mailingId = service.createMailing(name1, subject);
        assertNotNull(mailingId);

        mailingId2 = service.createMailing(name2, subject);
        assertNotNull(mailingId2);

        mailingId3 = service.createMailing(name3, subject);
        assertNotNull(mailingId3);


        List<String> fields = new ArrayList<>();
        fields.add(MailingFields.STATE);
        fields.add(MailingFields.TYPE);
        fields.add(MailingFields.NAME);
        fields.add(MailingFields.SCHEDULE_TIME);

        Page<Mailing> mailings = service.getMailingsBySubject(subject, MailingFilterOperators.STRING_OP_EQUALS, fields, 1, 100);

//        assertEquals(3, mailings.getTotalItems());
//        assertThat(mailings.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId))));
//        assertThat(mailings.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId2))));
//        assertThat(mailings.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId3))));
    }

    @Test
    public void testGetMailingsBySubjectStartsWith() throws MaileonException {
        String subject = getRandomSubject();
        String name1 = getRandomName();
        String name2 = getRandomName();
        String name3 = getRandomName();

        mailingId = service.createMailing(name1, subject);
        assertNotNull(mailingId);

        mailingId2 = service.createMailing(name2, subject);
        assertNotNull(mailingId2);

        mailingId3 = service.createMailing(name3, subject);
        assertNotNull(mailingId3);


        List<String> fields = new ArrayList<>();
        fields.add(MailingFields.STATE);
        fields.add(MailingFields.TYPE);
        fields.add(MailingFields.NAME);
        fields.add(MailingFields.SCHEDULE_TIME);
        System.out.println(subject.substring(0, 10));

        Page<Mailing> mailings = service.getMailingsBySubject(subject.substring(0, 10), MailingFilterOperators.STRING_OP_STARTS_WITH, fields, 1, 100);

//        assertEquals(3, mailings.getTotalItems());
//        assertThat(mailings.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId))));
//        assertThat(mailings.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId2))));
//        assertThat(mailings.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId3))));
    }

    @Test
    public void testGetMailingsBySubjectEndsWith() throws MaileonException {
        String subject = getRandomSubject();
        String name1 = getRandomName();
        String name2 = getRandomName();
        String name3 = getRandomName();

        mailingId = service.createMailing(name1, subject);
        assertNotNull(mailingId);

        mailingId2 = service.createMailing(name2, subject);
        assertNotNull(mailingId2);

        mailingId3 = service.createMailing(name3, subject);
        assertNotNull(mailingId3);


        List<String> fields = new ArrayList<>();
        fields.add(MailingFields.STATE);
        fields.add(MailingFields.TYPE);
        fields.add(MailingFields.NAME);
        fields.add(MailingFields.SCHEDULE_TIME);

        Page<Mailing> mailings = service.getMailingsBySubject(subject.substring(10, subject.length()), MailingFilterOperators.STRING_OP_ENDS_WITH, fields, 1, 100);

//        assertEquals(3, mailings.getTotalItems());
//        assertThat(mailings.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId))));
//        assertThat(mailings.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId2))));
//        assertThat(mailings.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId3))));
    }

    @Test
    public void testGetMailingsBySubjectContains() throws MaileonException {

        String subject = getRandomSubject();
        String name1 = getRandomName();
        String name2 = getRandomName();
        String name3 = getRandomName();

        mailingId = service.createMailing(name1, subject);
        assertNotNull(mailingId);

        mailingId2 = service.createMailing(name2, subject);
        assertNotNull(mailingId2);

        mailingId3 = service.createMailing(name3, subject);
        assertNotNull(mailingId3);


        List<String> fields = new ArrayList<>();
        fields.add(MailingFields.STATE);
        fields.add(MailingFields.TYPE);
        fields.add(MailingFields.NAME);
        fields.add(MailingFields.SCHEDULE_TIME);

        Page<Mailing> mailings = service.getMailingsBySubject(subject, MailingFilterOperators.STRING_OP_CONTAINS, fields, 1, 100);

        assertEquals(3, mailings.getTotalItems());
        assertThat(mailings.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId))));
        assertThat(mailings.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId2))));
        assertThat(mailings.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId3))));
    }

//    @Test
//    public void testGetMailingsBySchedulingTime() throws MaileonException {
//        String subject = getRandomSubject();
//        String name1 = getRandomName();
//
//        mailingId = service.createMailing(name1, subject);
//        assertNotNull(mailingId);
//
//
//
//        List<String> fields = new ArrayList<String>();
//        fields.add(MailingFields.STATE);
//        fields.add(MailingFields.TYPE);
//        fields.add(MailingFields.NAME);
//        fields.add(MailingFields.SCHEDULE_TIME);
//
//        String time = "2013-10-07 12:00:00";
//
//        Page<Mailing> mailings = service.getMailingsBySchedulingTime(time, false, fields, 3, 3);
//        System.out.format("Returned mailing page with page_index = %d, page_size=%d, number_pages=%d, mailings_total=%d\n", mailings.getPageIndex(), mailings.getPageSize(), mailings.getCountPages(), mailings.getTotalItems());
//        for (Mailing mailing:mailings.getItems()) {
//            System.out.println(mailing);
//        }
//    }

//    @Test
//    public void testGetMailingsByKeywords() throws MaileonException {
//        List<String> keywords = new ArrayList<String>();
//        keywords.add("test");
//
//
//
//        List<String> fields = new ArrayList<String>();
//        fields.add(MailingFields.STATE);
//        fields.add(MailingFields.TYPE);
//        fields.add(MailingFields.NAME);
//        fields.add(MailingFields.SCHEDULE_TIME);
//
//        Page<Mailing> mailings = service.getMailingsByKeywords(keywords, MailingFilterOperators.LIST_OP_CONTAINS_ALL, fields, 1, 5);
//        System.out.format("Returned mailing page with page_index = %d, page_size=%d, number_pages=%d, mailings_total=%d\n", mailings.getPageIndex(), mailings.getPageSize(), mailings.getCountPages(), mailings.getTotalItems());
//        for (Mailing mailing:mailings.getItems()) {
//            System.out.println(mailing);
//        }
//    }

//    @Test
//    public void testGetMailingsByState() throws MaileonException {
//        List<String> states = new ArrayList<String>();
//        states.add("queued");
//        states.add("draft");
//
//        List<String> fields = new ArrayList<String>();
//        fields.add(MailingFields.STATE);
//        fields.add(MailingFields.TYPE);
//        fields.add(MailingFields.NAME);
//        fields.add(MailingFields.SCHEDULE_TIME);
//
//        Page<Mailing> mailings = service.getMailingsByStates(states, fields, 1, 2);
//        System.out.format("Returned mailing page with page_index = %d, page_size=%d, number_pages=%d, mailings_total=%d\n", mailings.getPageIndex(), mailings.getPageSize(), mailings.getCountPages(), mailings.getTotalItems());
//        for (Mailing mailing:mailings.getItems()) {
//            System.out.println(mailing);
//        }
//    }

    @Test
    public void testGetMailingsByType() throws MaileonException {

        String subject = getRandomSubject();
        String name1 = getRandomName();
        String name2 = getRandomName();
        String name3 = getRandomName();

        mailingId = service.createMailing(name1, subject, MaileonMailingsService.MailingType.DOI);
        assertNotNull(mailingId);

        mailingId2 = service.createMailing(name2, subject, MaileonMailingsService.MailingType.TRIGGER);
        assertNotNull(mailingId2);

        mailingId3 = service.createMailing(name3, subject, MaileonMailingsService.MailingType.TRIGGER);
        assertNotNull(mailingId3);

        List<String> types = new ArrayList<>();
        types.add("doi");
        List<String> fields = new ArrayList<>();

        Page<Mailing> mailings = service.getMailingsByTypes(types, fields, 1, 1000);

        assertThat(mailings.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId))));


        types = new ArrayList<>();
        types.add("trigger");

        mailings = service.getMailingsByTypes(types, fields, 1, 1000);

        assertThat(mailings.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId2))));
        assertThat(mailings.getItems(), Matchers.<Mailing>hasItem(Matchers.<Mailing>hasProperty("id", is(mailingId3))));
    }

    /**
     * Check delete a mailing
     * @throws MaileonException
     */
    @Test
    public void testDeleteMailing() throws MaileonException {

        String subject = getRandomSubject();
        String name = getRandomName();

        mailingId = service.createMailing(name, subject);
        assertNotNull(mailingId);

        service.deleteMailing(mailingId);
    }

    /**
     * Check delete mailing that fails due to sealed state
     * @throws MaileonException
     */
    @Test (expected = MaileonException.class)
    public void testDeleteMailingFailing() throws MaileonException {
        try {
            service.deleteMailing(DEFAULT_MAILING_ID_SEALED);
            fail("Sealed mailing expected, should not be able to delete");
        } catch (MaileonException e) {
            assertEquals(409, e.getStatusCode());
            throw e;
        }
    }


    /**
     * Check setting and retrieving the sender alias
     * @throws MaileonException
     */
    @Test public void testSenderAlias() throws MaileonException {

        String subject = getRandomSubject();
        String name = getRandomName();

        String newValue = UUID.randomUUID().toString().substring(0, 7);

        mailingId = service.createMailing(name, subject);
        assertNotNull(mailingId);

        String apiValue = service.getSenderAlias(mailingId);
        assertNull(apiValue);

        service.setSenderAlias(mailingId, newValue);

        apiValue = service.getSenderAlias(mailingId);
        assertEquals(newValue, apiValue);
    }


    /**
     * Check setting and retrieving the recipient alias
     * @throws MaileonException
     */
    @Test
    public void testRecipientAlias() throws MaileonException {

        String subject = getRandomSubject();
        String name = getRandomName();

        String newValue = UUID.randomUUID().toString().substring(0, 7);

        mailingId = service.createMailing(name, subject);
        assertNotNull(mailingId);

        String apiValue = service.getRecipientAlias(mailingId);
        assertNull(apiValue);

        service.setRecipientAlias(mailingId, newValue);

        apiValue = service.getRecipientAlias(mailingId);
        assertEquals(newValue, apiValue);
    }


    /**
     * Check setting and retrieving the speed level
     * @throws MaileonException
     */
    @Test
    public void testSpeedLevel() throws MaileonException {

        String subject = getRandomSubject();
        String name = getRandomName();

        String newValue = "high";

        mailingId = service.createMailing(name, subject);
        assertNotNull(mailingId);

        String apiValue = service.getSpeedLevel(mailingId);
        assertEquals("medium", apiValue); // Default is medium

        service.setSpeedLevel(mailingId, newValue);

        apiValue = service.getSpeedLevel(mailingId);
        assertEquals(newValue, apiValue);
    }

    /**
     * Check setting and retrieving the tracking strategy
     * @throws MaileonException
     */
    @Test
    public void testTrackingStrategy() throws MaileonException {

        String subject = getRandomSubject();
        String name = getRandomName();

        String newValue = "none";

        mailingId = service.createMailing(name, subject);
        assertNotNull(mailingId);

        String apiValue = service.getTrackingStrategy(mailingId);
        assertEquals("highest-permission", apiValue); // Default is highest-permission

        service.setTrackingStrategy(mailingId, newValue);

        apiValue = service.getTrackingStrategy(mailingId);
        assertEquals(newValue, apiValue);
    }

    /**
     * Check setting and retrieving the sender email
     * @throws MaileonException
     */
    @Test public void testSender() throws MaileonException {
        String subject = getRandomSubject();
        String name = getRandomName();

        String newValue = UUID.randomUUID().toString().substring(0, 7);

        mailingId = service.createMailing(name, subject);
        assertNotNull(mailingId);

        String apiValue = service.getSenderAddress(mailingId);
        assertNull(apiValue);

        service.setSenderAddress(mailingId, newValue);

        apiValue = service.getSenderAddress(mailingId);
        assertTrue(apiValue.startsWith(newValue+"@"));
    }


    /**
     * Check setting and getting the subject
     * @throws MaileonException
     */
    @Test
    public void testSubject() throws MaileonException {
        String subject = getRandomSubject();
        String name = getRandomName();

        String newValue = UUID.randomUUID().toString().substring(0, 7);

        mailingId = service.createMailing(name, subject);
        assertNotNull(mailingId);

        String apiValue = service.getSubject(mailingId);
        assertEquals(subject, apiValue);

        service.setSubject(mailingId, newValue);

        apiValue = service.getSubject(mailingId);
        assertEquals(newValue, apiValue);
    }

    /**
     * Check setting the subject failing due to sealed state
     * @throws MaileonException
     */
    @Test (expected = MaileonException.class)
    public void testSubjectFailed() throws MaileonException {
        try {
            service.setSubject(DEFAULT_MAILING_ID_SEALED, getRandomSubject());
        } catch (MaileonException e) {
            assertEquals(409, e.getStatusCode());
            throw e;
        }
    }

    /**
     * Check retrieving the target group ID
     * @throws MaileonException
     */
    @Test
    public void testTargetGroupId() throws MaileonException {
        String subject = getRandomSubject();
        String name = getRandomName();

        long newValue = DEFAULT_NEW_TARGET_GROUP_ID;

        mailingId = service.createMailing(name, subject);
        assertNotNull(mailingId);

        try {
            service.getTargetGroupId(mailingId);
            fail("there should no target group being found.");
        } catch (MaileonNotFoundException e ) {
            assertEquals(404, e.getStatusCode());
        }

        service.setTargetGroupId(mailingId, newValue);

        long apiValue = service.getTargetGroupId(mailingId);
        assertEquals(newValue, apiValue);
    }


    /**
     * Check setting and retrieving the target group ID
     * @throws MaileonException
     */
    @Test
    public void testName() throws MaileonException {
        String subject = getRandomSubject();
        String name = getRandomName();

        String newValue = UUID.randomUUID().toString().substring(0, 7);

        mailingId = service.createMailing(name, subject);
        assertNotNull(mailingId);

        String apiValue = service.getName(mailingId);
        assertEquals(name, apiValue);

        service.setName(mailingId, newValue);

        apiValue = service.getName(mailingId);
        assertEquals(newValue, apiValue);
    }

    /**
     * Check if the mailing is sealed
     * @throws MaileonException
     */
    @Test
    public void testSendTestmail() throws MaileonException {
        service.sendTestEmail(DEFAULT_MAILING_ID, DEFAULT_TEST_RECIPIENT);
    }

    /**
     * Check if the mailing is sealed
     * @throws MaileonException
     */
    @Test
    public void testSendTestmailToNormalGroup() throws MaileonException {
        checkService.startTestSendoutCheck(DEFAULT_MAILING_ID, DEFAULT_NOT_TEST_TARGETGROUP);
    }

    /**
     * Check if the mailing is sealed
     * @throws MaileonException
     */
    @Test
    public void testSendTestmailToTestGroup() throws MaileonException {
        checkService.startTestSendoutCheck(DEFAULT_MAILING_ID, DEFAULT_TEST_TARGETGROUP);
    }


    /**
     * Check if the mailing is sealed
     * @throws MaileonException
     */
    @Test
    public void testIsSealed() throws MaileonException {
        assertTrue(service.isSealed(DEFAULT_MAILING_ID_SEALED));
    }

    private String getRandomName() {
        return "API_"+ UUID.randomUUID();
    }

    private String getRandomSubject() {
        return "Subject_"+ UUID.randomUUID();
    }
}
