package com.maileon.api.transactions;

import com.maileon.api.MaileonException;
import com.maileon.api.MaileonNotFoundException;
import com.maileon.api.Page;
import com.maileon.api.test.TestProperties;
import org.hamcrest.Matcher;
import org.hamcrest.Matchers;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

public class MaileonTransactionTypesTests {

    private static MaileonTransactionsService service;
    private static TestProperties testProperties;
    private static final String TEST_TYPE_NAME = "test_tx_type";

    @BeforeClass
    public static void setUp() throws MaileonException {
        testProperties = new TestProperties();
        service = new MaileonTransactionsService(testProperties.getMaileonConfiguration());
        service.setDebug(testProperties.isDebug());
    }

    @Before
    public void cleanupOldTestData() throws MaileonException {
        service.setDebug(false);
        service.deleteTransactionTypeByName(TEST_TYPE_NAME);
        service.setDebug(testProperties.isDebug());
    }

    @Test
    public void testGetTransactionTypesCount() throws MaileonException {
        int count = service.getTransactionTypesCount();
        assertThat(count, is(greaterThan(-1)));
    }

    @Test
    public void testTransactionTypesCountIncreases() throws MaileonException {
        int countBefore = service.getTransactionTypesCount();
        TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME);
        int countAfter = service.getTransactionTypesCount();
        assertThat(countAfter, is(countBefore + 1));
    }

    @Test
    public void testGetTransactionTypesReturnsAllItems() throws MaileonException {
        int pageIndex = 1;
        int pageSize = 100;
        Page<TransactionType> types = service.getTransactionTypes(pageIndex, pageSize);
        assertThat(types.getTotalItems(), is(equalTo(service.getTransactionTypesCount())));
    }

    @Test
    public void testGetTransactionTypes() throws MaileonException {
        TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME);

        boolean found = false;
        Matcher<Iterable<? super TransactionType>> matcher =
                hasItem(Matchers.<TransactionType> hasProperty("name", is(TEST_TYPE_NAME)));

        int numPages = 2;
        for (int i = 1; i <= numPages; i++) {
            Page<TransactionType> page = service.getTransactionTypes(i, 100);
            numPages = page.getCountPages();
            found = found || matcher.matches(page.getItems());
        }

        if (!found) {
            fail("Transaction type " + TEST_TYPE_NAME + " was not found in the list of transaction types.");
        }
    }

    @Test
    public void testGetTransactionType() throws MaileonException {
        TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME);
        long id = service.findTransactionTypeByName(TEST_TYPE_NAME).getId();
        TransactionType returnedType = service.getTransactionType(id);
        assertThat(returnedType.getName(), is(TEST_TYPE_NAME));
    }

    @Test
    public void testGetTransactionTypeByName() throws MaileonException {
        TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME);
        long id = service.findTransactionTypeByName(TEST_TYPE_NAME).getId();
        TransactionType returnedType = service.getTransactionType(id);
        assertThat(returnedType.getName(), is(TEST_TYPE_NAME));
    }

    @Test
    public void testCreateComplexTransactionType() throws MaileonException {

        long id = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                {"test_boolean", AttributeType.BOOLEAN, false},
                {"test_date", AttributeType.DATE, false},
                {"test_double", AttributeType.DOUBLE, false},
                {"test_float", AttributeType.FLOAT, false},
                {"test_integer", AttributeType.INTEGER, false},
                {"test_timestamp", AttributeType.TIMESTAMP, false},
                {"test_string", AttributeType.STRING, false},
                {"test_json", AttributeType.JSON, false},
                {"test_boolean_t", AttributeType.BOOLEAN, true},
                {"test_date_t", AttributeType.DATE, true},
                {"test_double_t", AttributeType.DOUBLE, true},
                {"test_float_t", AttributeType.FLOAT, true},
                {"test_integer_t", AttributeType.INTEGER, true},
                {"test_timestamp_t", AttributeType.TIMESTAMP, true},
                {"test_string_t", AttributeType.STRING, true},
                {"test_json_t", AttributeType.JSON, true}
        });
        TransactionType type = service.getTransactionType(id);

        assertThat(type.getName(), is(TEST_TYPE_NAME));
        assertThat(type.getAttributes(), allOf(
                hasItem(
                        allOf(
                                Matchers.<Attribute> hasProperty("id"),
                                Matchers.<Attribute> hasProperty("name", is("test_boolean")),
                                Matchers.<Attribute> hasProperty("type", is(AttributeType.BOOLEAN)),
                                Matchers.<Attribute> hasProperty("required", is(false)))),
                hasItem(
                        allOf(
                                Matchers.<Attribute> hasProperty("id"),
                                Matchers.<Attribute> hasProperty("name", is("test_date")),
                                Matchers.<Attribute> hasProperty("type", is(AttributeType.DATE)),
                                Matchers.<Attribute> hasProperty("required", is(false)))),
                hasItem(
                        allOf(
                                Matchers.<Attribute> hasProperty("id"),
                                Matchers.<Attribute> hasProperty("name", is("test_double")),
                                Matchers.<Attribute> hasProperty("type", is(AttributeType.DOUBLE)),
                                Matchers.<Attribute> hasProperty("required", is(false)))),
                hasItem(
                        allOf(
                                Matchers.<Attribute> hasProperty("id"),
                                Matchers.<Attribute> hasProperty("name", is("test_float")),
                                Matchers.<Attribute> hasProperty("type", is(AttributeType.FLOAT)),
                                Matchers.<Attribute> hasProperty("required", is(false)))),
                hasItem(
                        allOf(
                                Matchers.<Attribute> hasProperty("id"),
                                Matchers.<Attribute> hasProperty("name", is("test_integer")),
                                Matchers.<Attribute> hasProperty("type", is(AttributeType.INTEGER)),
                                Matchers.<Attribute> hasProperty("required", is(false)))),
                hasItem(
                        allOf(
                                Matchers.<Attribute> hasProperty("id"),
                                Matchers.<Attribute> hasProperty("name", is("test_timestamp")),
                                Matchers.<Attribute> hasProperty("type", is(AttributeType.TIMESTAMP)),
                                Matchers.<Attribute> hasProperty("required", is(false)))),
                hasItem(
                        allOf(
                                Matchers.<Attribute> hasProperty("id"),
                                Matchers.<Attribute> hasProperty("name", is("test_string")),
                                Matchers.<Attribute> hasProperty("type", is(AttributeType.STRING)),
                                Matchers.<Attribute> hasProperty("required", is(false)))),
                hasItem(
                        allOf(
                                Matchers.<Attribute> hasProperty("id"),
                                Matchers.<Attribute> hasProperty("name", is("test_json")),
                                Matchers.<Attribute> hasProperty("type", is(AttributeType.JSON)),
                                Matchers.<Attribute> hasProperty("required", is(false)))),
                hasItem(
                        allOf(
                                Matchers.<Attribute> hasProperty("id"),
                                Matchers.<Attribute> hasProperty("name", is("test_boolean_t")),
                                Matchers.<Attribute> hasProperty("type", is(AttributeType.BOOLEAN)),
                                Matchers.<Attribute> hasProperty("required", is(true)))),
                hasItem(
                        allOf(
                                Matchers.<Attribute> hasProperty("id"),
                                Matchers.<Attribute> hasProperty("name", is("test_date_t")),
                                Matchers.<Attribute> hasProperty("type", is(AttributeType.DATE)),
                                Matchers.<Attribute> hasProperty("required", is(true)))),
                hasItem(
                        allOf(
                                Matchers.<Attribute> hasProperty("id"),
                                Matchers.<Attribute> hasProperty("name", is("test_double_t")),
                                Matchers.<Attribute> hasProperty("type", is(AttributeType.DOUBLE)),
                                Matchers.<Attribute> hasProperty("required", is(true)))),
                hasItem(
                        allOf(
                                Matchers.<Attribute> hasProperty("id"),
                                Matchers.<Attribute> hasProperty("name", is("test_float_t")),
                                Matchers.<Attribute> hasProperty("type", is(AttributeType.FLOAT)),
                                Matchers.<Attribute> hasProperty("required", is(true)))),
                hasItem(
                        allOf(
                                Matchers.<Attribute> hasProperty("id"),
                                Matchers.<Attribute> hasProperty("name", is("test_integer_t")),
                                Matchers.<Attribute> hasProperty("type", is(AttributeType.INTEGER)),
                                Matchers.<Attribute> hasProperty("required", is(true)))),
                hasItem(
                        allOf(
                                Matchers.<Attribute> hasProperty("id"),
                                Matchers.<Attribute> hasProperty("name", is("test_timestamp_t")),
                                Matchers.<Attribute> hasProperty("type", is(AttributeType.TIMESTAMP)),
                                Matchers.<Attribute> hasProperty("required", is(true)))),
                hasItem(
                        allOf(
                                Matchers.<Attribute> hasProperty("id"),
                                Matchers.<Attribute> hasProperty("name", is("test_string_t")),
                                Matchers.<Attribute> hasProperty("type", is(AttributeType.STRING)),
                                Matchers.<Attribute> hasProperty("required", is(true)))),
                hasItem(
                        allOf(
                                Matchers.<Attribute> hasProperty("id"),
                                Matchers.<Attribute> hasProperty("name", is("test_json_t")),
                                Matchers.<Attribute> hasProperty("type", is(AttributeType.JSON)),
                                Matchers.<Attribute> hasProperty("required", is(true))))));
    }

    @Test
    public void testCreateTransactionTypeWithArchivalDuration() throws MaileonException {

        long id = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, 7);
        TransactionType type = service.getTransactionType(id);

        assertThat(type.getName(), is(TEST_TYPE_NAME));
        assertThat(type.getArchivingDuration(), is(7));

    }

    @Test
    public void testCreateTransactionTypeWithoutArchivalDuration() throws MaileonException {

        long id = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME);
        TransactionType type = service.getTransactionType(id);

        assertThat(type.getName(), is(TEST_TYPE_NAME));
        assertEquals(type.getArchivingDuration(), null); // Best would be to check if the tag <archivalDuration> is not contained in the xml at all
    }

    public void testDeleteTransactionType() throws MaileonException {
        long id = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME);
        service.getTransactionType(id);
        service.deleteTransactionType(id);
    }

    @Test(expected=MaileonNotFoundException.class)
    public void testDeleteTransactionTypeCheckDelete() throws MaileonException {
        long id = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME);
        service.deleteTransactionType(id);
        service.getTransactionType(id); // After deletion = 404
    }

}
