package com.maileon.api.transactions;

import com.maileon.api.MaileonBadRequestException;
import com.maileon.api.MaileonException;
import com.maileon.api.contacts.Contact;
import com.maileon.api.contacts.MaileonContactsService;
import com.maileon.api.contacts.Permission;
import com.maileon.api.contacts.SynchronizationMode;
import com.maileon.api.test.TestProperties;
import org.hamcrest.Matchers;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;

import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.*;
import static org.junit.Assert.*;

public class MaileonTransactionsTest {

    private static final String TEST_TYPE_NAME = "test_type2";
    private static final String TEST_EMAIL1 = "isabell.integrationstest@xqueue.com";
    private static final String TEST_EXTERNALID1 = "integrationtest-001";
    private static final String TEST_EMAIL2 = "ingrid.integrationstest@xqueue.com";
    private static final String TEST_EXTERNALID2 = "integrationtest-002";
    private static final String TEST_EMAIL3 = "doris.duplicateemail@xqueue.com";
    private static final String TEST_EXTERNALID3_1 = "integrationtest-001";
    private static final String TEST_EXTERNALID3_2 = "integrationtest-002";
    private static final String TEST_WMAIL4 = "norbert.notexisting@xqueue.com";
    private static MaileonTransactionsService service;
    private static MaileonContactsService contactsService;
    private static TestProperties testProperties;


    @BeforeClass
    public static void setUp() throws MaileonException {
        testProperties = new TestProperties();
        service = new MaileonTransactionsService(testProperties.getMaileonConfiguration());
        service.setDebug(testProperties.isDebug());
        contactsService = new MaileonContactsService(testProperties.getMaileonConfiguration());
        contactsService.setDebug(testProperties.isDebug());
        createTestContacts();
    }

    private static void createTestContacts() {
        try {
            Contact contact = new Contact();
            contact.setAnonymous(false);
            contact.setEmail(TEST_EMAIL1);
            contact.setExternalId(TEST_EXTERNALID1);
            contact.setPermission(Permission.DOI);
            contactsService.createContact(contact, SynchronizationMode.UPDATE);

            contact = new Contact();
            contact.setAnonymous(false);
            contact.setEmail(TEST_EMAIL2);
            contact.setExternalId(TEST_EXTERNALID2);
            contact.setPermission(Permission.DOI);
            contactsService.createContact(contact, SynchronizationMode.UPDATE);

            contact = new Contact();
            contact.setAnonymous(false);
            contact.setEmail(TEST_EMAIL3);
            contact.setExternalId(TEST_EXTERNALID3_1);
            contact.setPermission(Permission.DOI);
            contactsService.createContact(contact, SynchronizationMode.UPDATE);

            contact= new Contact();
            contact.setAnonymous(false);
            contact.setEmail(TEST_EMAIL3);
            contact.setExternalId(TEST_EXTERNALID3_2);
            contact.setPermission(Permission.DOI);
            contactsService.createContact(contact, SynchronizationMode.UPDATE);

        } catch (MaileonException e) {
            throw new RuntimeException(e);
        }
    }

    @Before
    public void cleanupOldTestData() throws MaileonException {
        service.setDebug(false);
        service.deleteTransactionTypeByName(TEST_TYPE_NAME);
        service.setDebug(testProperties.isDebug());
    }

    @Test
    public void testCreateSingleTransaction() throws MaileonException {
        long typeId = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                {"name", AttributeType.STRING, true},
                {"total", AttributeType.DOUBLE, true}
        });

        String content = "{\"name\": \"Max Mustermann\", \"total\": 235.29}";
        Transaction transaction = buildTransaction(typeId, TEST_EMAIL1, content);
        TransactionProcessingReport report = service.createTransaction(transaction, true, false);

        assertEquals(report.getContact().getEmail(), TEST_EMAIL1);
        assertNotNull(report.getContact().getId());
        assertTrue(report.isQueued());
    }

    @Test
    public void testCreateSingleTransactionWithImport() throws MaileonException {
        long typeId = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                {"name", AttributeType.STRING, true},
                {"total", AttributeType.DOUBLE, true}
        });

        String content = "{\"name\": \"Max Mustermann\", \"total\": 235.29}";
        Transaction transaction = buildImportTransaction(typeId, TEST_WMAIL4, content);
        TransactionProcessingReport report = service.createTransaction(transaction, true, false);

        assertEquals(report.getContact().getEmail(), TEST_WMAIL4);
        assertNotNull(report.getContact().getId());
        assertTrue(report.isQueued());
    }

    @Test
    public void testCreateSingleTransactionAsList() throws MaileonException {
        long typeId = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                {"name", AttributeType.STRING, true},
                {"total", AttributeType.DOUBLE, true}
        });

        String content = "{\"name\": \"Max Mustermann\", \"total\": 235.29}";
        Transaction transaction = buildTransaction(typeId, TEST_EMAIL1, content);
        List<TransactionProcessingReport> reports = service.createTransactions(Arrays.asList(transaction), false);

        assertEquals(reports.size(), 1);
        assertEquals(reports.get(0).getContact().getEmail(), TEST_EMAIL1);
        assertNotNull(reports.get(0).getContact().getId());
        assertTrue(reports.get(0).isQueued());
    }

    @Test
    public void testCreateSingleTransactionWithAttachment() throws MaileonException, IOException {
        long typeId = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                {"name", AttributeType.STRING, true},
                {"total", AttributeType.DOUBLE, true}
        });
        String content = "{\"name\": \"Max Mustermann\", \"total\": 235.29}";
        Transaction transaction = buildTransaction(typeId, TEST_EMAIL1, content);
        transaction.addAttachment("awesome.png", "image/png", getClass().getResourceAsStream("/awesome.png"));
        TransactionProcessingReport report = service.createTransaction(transaction, false);

        assertEquals(report.getContact().getEmail(), TEST_EMAIL1);
        assertNotNull(report.getContact().getId());
        assertTrue(report.isQueued());
    }

    @Test(expected=MaileonBadRequestException.class)
    public void testCreateTransactionRequiredAttributeMissingFails() throws MaileonException {
        long typeId = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                {"name", AttributeType.STRING, true},
                {"total", AttributeType.DOUBLE, true}
        });
        String content = "{\"total\": 235.29}";
        Transaction transaction = buildTransaction(typeId, TEST_EMAIL1, content);
        service.createTransaction(transaction, false);
    }

    @Test
    public void testCreateTransactionIgnoreInvalid() throws MaileonException {
        long typeId = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                {"name", AttributeType.STRING, true},
                {"total", AttributeType.DOUBLE, true}
        });
        String content = "{\"total\": 235.29}";
        Transaction transaction = buildTransaction(typeId, TEST_EMAIL1, content);
        TransactionProcessingReport report = service.createTransaction(transaction, true);
        assertThat(report.isQueued(), is(false));
        assertThat(report.getMessage(), allOf(containsString("required"), containsString("name")));
    }

    @Test
    public void testCreateTransactionWithNestedAttributes() throws MaileonException {
        long typeId = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                {"customer.name", AttributeType.STRING, true},
                {"total", AttributeType.DOUBLE, true}
        });
        String content = "{\"total\": 235.29, \"customer\": {\"name\": \"Max Mustermann\"}}";
        Transaction transaction = buildTransaction(typeId, TEST_EMAIL1, content);
        TransactionProcessingReport report = service.createTransaction(transaction, false);

        assertEquals(report.getContact().getEmail(), TEST_EMAIL1);
        assertNotNull(report.getContact().getId());
        assertTrue(report.isQueued());
    }

    @Test
    public void testCreateMultipleTransactions() throws MaileonException {
        long typeId = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                {"customer.name", AttributeType.STRING, true},
                {"total", AttributeType.DOUBLE, true}
        });
        Transaction transaction1 = buildTransaction(typeId, TEST_EMAIL1, "{\"total\": 235.29, \"customer\": {\"name\": \"Max Mustermann\"}}");
        Transaction transaction2 = buildTransaction(typeId, TEST_EMAIL1, "{\"total\": 120.99, \"customer\": {\"name\": \"Max Mustermann\"}}");
        List<TransactionProcessingReport> reports = service.createTransactions(Arrays.asList(transaction1, transaction2), false);

        assertEquals(reports.size(), 2);
        assertThat(reports, allOf(
                hasItem(
                        allOf(
                                Matchers.<TransactionProcessingReport>hasProperty("contact",
                                        allOf(
                                                Matchers.<ContactReference>hasProperty("id"),
                                                Matchers.<ContactReference>hasProperty("email", is(TEST_EMAIL1)))),
                                Matchers.<TransactionProcessingReport>hasProperty("queued", is(true)))),
                hasItem(
                        allOf(
                                Matchers.<TransactionProcessingReport>hasProperty("contact",
                                        allOf(
                                                Matchers.<ContactReference>hasProperty("id"),
                                                Matchers.<ContactReference>hasProperty("email", is(TEST_EMAIL1)))),
                                Matchers.<TransactionProcessingReport>hasProperty("queued", is(true))))
        ));

    }

    @Test
    public void testCreateTransactionsTwoEmailContacts() throws MaileonException {
        long typeId = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                {"customer.name", AttributeType.STRING, true},
                {"total", AttributeType.DOUBLE, true}
        });
        Transaction transaction1 = buildTransaction(typeId, TEST_EMAIL1, "{\"total\": 235.29, \"customer\": {\"name\": \"Max Mustermann\"}}");
        Transaction transaction2 = buildTransaction(typeId, TEST_EMAIL1, "{\"total\": 120.99, \"customer\": {\"name\": \"Max Mustermann\"}}");
        transaction1.getContact().setEmail(TEST_EMAIL1);
        transaction2.getContact().setEmail(TEST_EMAIL2);
        List<TransactionProcessingReport> reports = service.createTransactions(Arrays.asList(transaction1, transaction2), false);

        assertEquals(reports.size(), 2);
        assertThat(reports, allOf(
                hasItem(
                        allOf(
                                Matchers.<TransactionProcessingReport>hasProperty("contact",
                                        allOf(
                                                Matchers.<ContactReference>hasProperty("id"),
                                                Matchers.<ContactReference>hasProperty("email", is(TEST_EMAIL1)))),
                                Matchers.<TransactionProcessingReport>hasProperty("queued", is(true)))),
                hasItem(
                        allOf(
                                Matchers.<TransactionProcessingReport>hasProperty("contact",
                                        allOf(
                                                Matchers.<ContactReference>hasProperty("id"),
                                                Matchers.<ContactReference>hasProperty("email", is(TEST_EMAIL2)))),
                                Matchers.<TransactionProcessingReport>hasProperty("queued", is(true))))
        ));
    }

    @Test
    public void testCreateTransactionsTwoExternalIdContacts() throws MaileonException {
        long typeId = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                {"customer.name", AttributeType.STRING, true},
                {"total", AttributeType.DOUBLE, true}
        });
        Transaction transaction1 = buildTransaction(typeId, TEST_EMAIL1, "{\"total\": 235.29, \"customer\": {\"name\": \"Max Mustermann\"}}");
        Transaction transaction2 = buildTransaction(typeId, TEST_EMAIL2, "{\"total\": 120.99, \"customer\": {\"name\": \"Max Mustermann\"}}");
        transaction1.getContact().setEmail(null);
        transaction1.getContact().setExternalId(TEST_EXTERNALID1);
        transaction2.getContact().setEmail(null);
        transaction2.getContact().setExternalId(TEST_EXTERNALID2);
        List<TransactionProcessingReport> reports = service.createTransactions(Arrays.asList(transaction1, transaction2), false);

        assertEquals(reports.size(), 2);
        assertThat(reports, allOf(
                hasItem(
                        allOf(
                                Matchers.<TransactionProcessingReport>hasProperty("contact",
                                        allOf(
                                                Matchers.<ContactReference>hasProperty("id"),
                                                Matchers.<ContactReference>hasProperty("externalId", is(TEST_EXTERNALID1)))),
                                Matchers.<TransactionProcessingReport>hasProperty("queued", is(true)))),
                hasItem(
                        allOf(
                                Matchers.<TransactionProcessingReport>hasProperty("contact",
                                        allOf(
                                                Matchers.<ContactReference>hasProperty("id"),
                                                Matchers.<ContactReference>hasProperty("externalId", is(TEST_EXTERNALID2)))),
                                Matchers.<TransactionProcessingReport>hasProperty("queued", is(true))))
        ));
    }

    @Test
    public void testCreateTransactionsTwoExternalIdContactsAndSameEmail() throws MaileonException {
        long typeId = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                {"customer.name", AttributeType.STRING, true},
                {"total", AttributeType.DOUBLE, true}
        });
        Transaction transaction1 = buildTransaction(typeId, TEST_EMAIL3, "{\"total\": 235.29, \"customer\": {\"name\": \"Max Mustermann\"}}");
        Transaction transaction2 = buildTransaction(typeId, TEST_EMAIL3, "{\"total\": 120.99, \"customer\": {\"name\": \"Max Mustermann\"}}");
        transaction1.getContact().setEmail(null);
        transaction1.getContact().setExternalId(TEST_EXTERNALID3_1);
        transaction2.getContact().setEmail(null);
        transaction2.getContact().setExternalId(TEST_EXTERNALID3_2);
        List<TransactionProcessingReport> reports = service.createTransactions(Arrays.asList(transaction1, transaction2), false);

        assertEquals(reports.size(), 2);
        assertThat(reports, allOf(
                hasItem(
                        allOf(
                                Matchers.<TransactionProcessingReport>hasProperty("contact",
                                        allOf(
                                                Matchers.<ContactReference>hasProperty("id"),
                                                Matchers.<ContactReference>hasProperty("externalId"))),
                                Matchers.<TransactionProcessingReport>hasProperty("queued", is(true)))),
                hasItem(
                        allOf(
                                Matchers.<TransactionProcessingReport>hasProperty("contact",
                                        allOf(
                                                Matchers.<ContactReference>hasProperty("id"),
                                                Matchers.<ContactReference>hasProperty("externalId"))),
                                Matchers.<TransactionProcessingReport>hasProperty("queued", is(true))))
        ));
    }

    @Test(expected=MaileonBadRequestException.class)
    public void testCreateTransactionFailsIfJSONTooLong() throws MaileonException {
        long typeId = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                {"content", AttributeType.JSON, true}
        });
        String content = "{\"content\": " + makeLongJSONArray() + "}";
        Transaction transaction = buildTransaction(typeId, TEST_EMAIL1, content);
        service.createTransaction(transaction, false);
    }

    @Test(expected=MaileonBadRequestException.class)
    public void testCreateTransactionFailsIfStringTooLong() throws MaileonException {
        long typeId = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                {"content", AttributeType.STRING, true}
        });
        String content = "{\"content\": \"" + makeLongString() + "\"}";
        Transaction transaction = buildTransaction(typeId, TEST_EMAIL1, content);
        service.createTransaction(transaction, false);
    }

    private String makeLongString() {
        StringBuilder longCat = new StringBuilder();
        do {
            longCat.append("nyanyanya");
        } while (longCat.length() < 1000);
        return longCat.toString();
    }

    private String makeLongJSONArray() {
        StringBuilder longCat = new StringBuilder();
        boolean first = true;
        do {
            if (!first) {
                longCat.append(",");
            }
            first = false;
            longCat.append("\"nyanyanya\"");
        } while (longCat.length() < 64000 - 2);
        return "[" + longCat.toString() + "]";
    }

    @Test
    public void testDeleteTransactions() throws MaileonException {
        long typeId = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                {"customer.name", AttributeType.STRING, true},
                {"total", AttributeType.DOUBLE, true}
        });
        System.out.println(typeId);
        Transaction transaction1 = buildTransaction(typeId, TEST_EMAIL1, "{\"total\": 235.29, \"customer\": {\"name\": \"Max Mustermann\"}}");
        Transaction transaction2 = buildTransaction(typeId, TEST_EMAIL1, "{\"total\": 120.99, \"customer\": {\"name\": \"Max Mustermann\"}}");
        service.createTransactions(Arrays.asList(transaction1, transaction2), false);
        service.deleteTransactions(typeId, Long.MAX_VALUE);
    }

    @Test
    public void testOptionalBlankStringsIgnored() throws MaileonException {
        long typeId = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                {"customer.name", AttributeType.STRING, false},
                {"garbage", AttributeType.STRING, false}
        });
        Transaction transaction = buildTransaction(typeId, TEST_EMAIL1, "{\"customer.name\": \" \n\r\0\", \"garbage\": \"   \"}");
        TransactionProcessingReport report = service.createTransaction(transaction, false);
        assertThat(report.isQueued(), is(true));
    }

    @Test
    public void testRequiredBlankStringsThrowError() throws MaileonException {
        try {
            long typeId = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                    {"customer.name", AttributeType.STRING, true},
                    {"garbage", AttributeType.STRING, true}
            });
            Transaction transaction1 = buildTransaction(typeId, TEST_EMAIL1, "{\"customer.name\": \" \n\r\0\", \"garbage\": \"   \"}");
            service.createTransaction(transaction1, true, false);
            fail("MaileonBadRequestException expected.");
        } catch (MaileonBadRequestException e) {
            assertThat(e.getMessage(), containsString("A null or empty value was provided for the required attribute"));
        }
    }

    @Test
    public void testNoFloatingPointAcceptedAsInteger() throws MaileonException {
        try {
            long typeId = TransactionsTestUtils.createTransactionType(service, TEST_TYPE_NAME, new Object[][]{
                    {"number_of_children", AttributeType.INTEGER, false}
            });
            Transaction transaction1 = buildTransaction(typeId, TEST_EMAIL1, "{\"number_of_children\": 2.38}");
            service.createTransaction(transaction1, true, false);
            fail("MaileonBadRequestException expected.");
        } catch (MaileonBadRequestException e) {
            assertThat(e.getMessage(), containsString("integer expected"));
        }
    }

    private Transaction buildTransaction(long typeId, String email, String content) {
        Transaction transaction = new Transaction();
        transaction.setType(typeId);
        ContactReference contact = new ContactReference();
        contact.setEmail(email);
        transaction.setContact(contact);
        transaction.setContentFromString(content);
        return transaction;
    }

    private Transaction buildImportTransaction(long typeId, String email, String content) {

        // Create the transaction object
        Transaction transaction = new Transaction();

        // Assign the type ID
        transaction.setType(typeId);

        // Create the import statement
        ImportReference importReference = new ImportReference();

        // Add the contact to the import statement
        ImportContactReference contact = new ImportContactReference();
        contact.setEmail(email);
        contact.setPermission(Permission.DOI_PLUS.getCode());
        importReference.setContact(contact);

        // Set import statement and content to the transaction
        transaction.setImportReference(importReference);
        transaction.setContentFromString(content);
        return transaction;
    }

}
